/*****************************************************************************************************************************
--
--    File Name    : Display_Enhancement_tb.v 

--    Description  : This module provides the test environment for Image Enhancement IP.

--                   For testing this IP, inputs should come from RGB to YCbCr IP and then outputs should be 
					 connected to YCbCr to RGB IP.
					 
					 The following file is required for the simulation :
					 RGB_in.txt -->  It contains hex values which are in the following format: 
									 First 2 bytes width of the image, next 2 bytes height of the image.
									 Then follows B,G and R values.
									 For more details visit Microsemi Video Web Page.

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module display_enhancement_test;
  /************************************************************************
          Parameters 
  *************************************************************************/
  parameter HEIGHT = 720;
  parameter WIDTH = 1280;
  parameter CLKPERIOD = 13.46;
  parameter WRITE_CLK = 12.5;
  parameter WAIT = 1;//delay between two lines of input
  parameter DATA_BIT_WIDTH = 24; //pixel width
  parameter IMAGE_FILE_NAME = "RGB_in.txt"; 
  //Parameters from the DUT
  parameter g_RGB_DATA_BIT_WIDTH = 8;
  parameter g_YCbCr_DATA_BIT_WIDTH = 8;
  parameter g_B_C_SAT_CONSTANT_WIDTH = 8;
  parameter g_HUE_CONSTANT_WIDTH = 9;
  parameter g_SIN_COS_DIN_WIDTH = 10;
  parameter g_SIN_COS_MEM_DEPTH = 180;
  parameter g_SIN_COS_DWIDTH = 10;
  localparam MAX_SIZE = (HEIGHT * WIDTH * (DATA_BIT_WIDTH / 8)) + 4;
  localparam MAX_SIZE_24 = (HEIGHT * WIDTH) + 1;
  /************************************************************************
          Inputs and Outputs to the DUT 
  *************************************************************************/
  reg Clock;
  reg Reset;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Red;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Green;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Blue;
  reg dataValidIn_rgb;
  reg H_sync_in;
  reg V_sync_in;
  reg H_active_in;
  reg V_active_in;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Y_out;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cb_out;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cr_out;
  wire dataValidOut_rgb;
  wire H_sync_out;
  wire V_sync_out;
  wire H_active_out;
  wire V_active_out;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] image_8[MAX_SIZE - 1 : 0];
  reg[DATA_BIT_WIDTH - 1 : 0] image_24[MAX_SIZE_24 - 1 : 0];
  integer i,
          j_rgb;
  reg[31 : 0] counter_h_rgb,
              counter_v_rgb,
              counter_h_in_rgb,
              counter_v_in_rgb;
  reg w_clk;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] brightness;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] contrast;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] saturation;
  reg signed[(g_HUE_CONSTANT_WIDTH - 1) : 0] hue;
  reg sin_write_en_i;
  reg cos_write_en_i;
  reg[7 : 0] sin_write_addr_i;
  reg[7 : 0] cos_write_addr_i;
  reg[(g_SIN_COS_DIN_WIDTH - 1) : 0] sin_value_i;
  reg[(g_SIN_COS_DIN_WIDTH - 1) : 0] cos_value_i;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Y_i;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cb_i;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cr_i;
  reg dataValidIn_bright;
  reg horz_sync_i;
  reg vert_sync_i;
  reg hactive_i;
  reg vactive_i;
  wire horz_sync_o;
  wire vert_sync_o;
  wire hactive_o;
  wire vactive_o;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Yout;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cbout;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Crout;
  wire dataValidOut_bright;
  reg[31 : 0] counter_h_bright,
              counter_v_bright,
              counter_h_in_bright,
              counter_v_in_bright;
  wire write_en;
  wire[7 : 0] write_addr;
  wire[(g_SIN_COS_DWIDTH - 1) : 0] sin_value_o;
  wire[(g_SIN_COS_DWIDTH - 1) : 0] cos_value_o;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Y;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cb;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cr;
  reg dataValidIn_y2r;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Red_out;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Green_out;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Blue_out;
  wire dataValidOut_y2r;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_B[MAX_SIZE_24 - 2 : 0];
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_G[MAX_SIZE_24 - 2 : 0];
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_R[MAX_SIZE_24 - 2 : 0];
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_final[MAX_SIZE - 5 : 0];
  integer k_y2r,
          l,
          m;
  reg[31 : 0] counter_h_y2r,
              counter_v_y2r,
              counter_h_in_y2r,
              counter_v_in_y2r;
  initial
    begin
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>            DISPLAY ENHANCEMENT IP SIMULATION STARTED                       <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      Reset = 0;
      Clock = 0;
      H_sync_in = 0;
      V_sync_in = 0;
      H_active_in = 0;
      V_active_in = 0;
      Red = 0;
      Green = 0;
      Blue = 0;
      dataValidIn_rgb = 0;
      j_rgb = 1;
      counter_v_in_rgb = 0;
      counter_h_in_rgb = 0;
      counter_v_rgb = 0;
      counter_h_rgb = 0;
      w_clk = 0;
      Y_i = 0;
      Cb_i = 0;
      Cr_i = 0;
      dataValidIn_bright = 0;
      horz_sync_i = 0;
      vert_sync_i = 0;
      hactive_i = 0;
      vactive_i = 0;
      /************************************************************************/
      // Change these constants according to the limits mentioned in user guide 
      brightness = 8'd10;
      saturation = 8'd20;
      contrast = 8'd30;
      hue = 9'd5;
      /*************************************************************************/
      counter_v_bright = 0;
      counter_h_bright = 0;
      counter_v_in_bright = 0;
      counter_h_in_bright = 0;
      Y = 0;
      Cb = 0;
      Cr = 0;
      k_y2r = 0;
      l = 0;
      m = 0;
      counter_v_y2r = 0;
      counter_h_y2r = 0;
      counter_v_in_y2r = 0;
      counter_h_in_y2r = 0;
    end
  /************************************************************************
          Generates Clock signal
  *************************************************************************/
  initial
    begin
      forever
        #(CLKPERIOD / 2)
        Clock = ~Clock;
    end
  initial
    begin
      forever
        #(WRITE_CLK / 2)
        w_clk = ~w_clk;
    end
  /************************************************************************
          RESET : Generates a Reset pulse 
  *************************************************************************/
  initial
    begin
      Reset = 1;
      #6
      Reset = 0;
      #6
      Reset = 1;
    end
  /************************************************************************
          Read the input image 
  *************************************************************************/
  initial
    begin
      $readmemh(IMAGE_FILE_NAME, image_8);
      for (i = 0; i < MAX_SIZE_24; i = i + 1)
        begin
          image_24[i] = (({ 16'b0, image_8[i * 3 + 1] } << 16) | ({ 16'b0, image_8[i * 3 + 2] } << 8) | { image_8[i * 3 + 3] });
        end
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                         IMAGE LOADED                                       <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
    end
  /************************************************************************
  *********************RGB to YCbCr Conversion Starts**********************
  *************************************************************************/
  /************************************************************************
  Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (counter_h_rgb == (WIDTH - 1))
            begin
              counter_h_rgb = 0;
              counter_v_rgb = counter_v_rgb + 1;
            end
          else
            if (dataValidOut_rgb)
              counter_h_rgb = counter_h_rgb + 1;
        end
    end
  /************************************************************************
  Debug Signals: Input horizontal counter and input vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(counter_h_in_rgb)
        begin
          if (counter_h_in_rgb == (WIDTH))
            begin
              counter_v_in_rgb = counter_v_in_rgb + 1;
            end
          else
            counter_v_in_rgb = counter_v_in_rgb;
        end
    end
  /**************************************************************************
  Task to give the input (image) line after line with some delay between them
  ***************************************************************************/
  task read_image_delay_after_each_line_rgb;
    begin
      if (counter_v_in_rgb == (HEIGHT))
        begin
          dataValidIn_rgb = 0;
        end
      else
        begin
          if ((counter_h_in_rgb == (WIDTH)) && (Reset == 1))
            begin
              dataValidIn_rgb = 0;
              counter_h_in_rgb = 0;
              wait_for_next_line_rgb();
            end
          else
            begin
              if ((counter_h_in_rgb < (WIDTH)) && (Reset == 1))
                begin
                  dataValidIn_rgb = 1;
                  Red = image_24[j_rgb][7 : 0];
                  Green = image_24[j_rgb][15 : 8];
                  Blue = image_24[j_rgb][23 : 16];
                  j_rgb = j_rgb + 1;
                  counter_h_in_rgb = counter_h_in_rgb + 1;
                end
            end
        end
    end
  endtask
  /************************************************************************
  Task to define the amount of delay between two lines of input (image)
  *************************************************************************/
  task wait_for_next_line_rgb;
    begin
      repeat (WAIT)
        @(posedge Clock)
        ;
    end
  endtask
  initial
    begin
      forever
        @(posedge Clock)
        begin
          dataValidIn_bright = dataValidOut_rgb;
        end
    end
  /************************************************************************
  *************************Display Enhancement Starts**********************
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (!Reset)
            begin
              sin_write_en_i <= 'd0;
              cos_write_en_i <= 'd0;
              sin_write_addr_i <= 'd0;
              cos_write_addr_i <= 'd0;
              sin_value_i <= 'd0;
              cos_value_i <= 'd0;
            end
          else
            begin
              sin_write_en_i <= write_en;
              cos_write_en_i <= write_en;
              sin_write_addr_i <= write_addr;
              cos_write_addr_i <= write_addr;
              sin_value_i <= sin_value_o;
              cos_value_i <= cos_value_o;
            end
        end
    end
  /************************************************************************
  Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (counter_h_bright == (WIDTH))
            begin
              counter_h_bright = 0;
              counter_v_bright = counter_v_bright + 1;
            end
          else
            if (dataValidOut_bright)
              counter_h_bright = counter_h_bright + 1;
        end
    end
  /************************************************************************
  Debug Signals: Input horizontal counter and input vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(counter_h_in_bright)
        begin
          if (counter_h_in_bright == (WIDTH))
            begin
              counter_v_in_bright = counter_v_in_bright + 1;
            end
          else
            counter_v_in_bright = counter_v_in_bright;
        end
    end
  /**************************************************************************
      Task to give YCbCr as input to Display Enhancement IP
  ***************************************************************************/
  task read_image_delay_after_each_line_bright;
    begin
      if ((counter_h_in_bright == (WIDTH)) && (Reset == 1))
        counter_h_in_bright = 0;
      else
        begin
          if ((counter_h_in_bright < (WIDTH)) && (Reset == 1) && dataValidOut_rgb)
            begin
              Y_i = Y_out;
              Cb_i = Cb_out;
              Cr_i = Cr_out;
              counter_h_in_bright = counter_h_in_bright + 1;
            end
        end
    end
  endtask
  initial
    begin
      forever
        @(posedge Clock)
        begin
          dataValidIn_y2r = dataValidOut_bright;
        end
    end
  /************************************************************************
  *********************YCbCr to RGB Conversion Starts**********************
  *************************************************************************/
  /************************************************************************
  Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (counter_h_y2r == (WIDTH))
            begin
              counter_h_y2r = 0;
              counter_v_y2r = counter_v_y2r + 1;
            end
          else
            if (dataValidOut_y2r)
              counter_h_y2r = counter_h_y2r + 1;
        end
    end
  /************************************************************************
  Debug Signals: Input horizontal counter and input vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(counter_h_in_y2r)
        begin
          if (counter_h_in_y2r == (WIDTH))
            begin
              counter_v_in_y2r = counter_v_in_y2r + 1;
            end
          else
            counter_v_in_y2r = counter_v_in_y2r;
        end
    end
  /**************************************************************************
   Task to convert data in YCbCr space to RGB
  ***************************************************************************/
  task read_image_delay_after_each_line_y2r;
    begin
      if (counter_v_in_y2r == (HEIGHT))
        begin
          done_processing;
        end
      else
        begin
          if ((counter_h_in_y2r == (WIDTH)) && (Reset == 1))
            counter_h_in_y2r = 0;
          else
            begin
              if ((counter_h_in_y2r < (WIDTH)) && (Reset == 1) && dataValidOut_bright)
                begin
                  Y = Yout;
                  Cb = Cbout;
                  Cr = Crout;
                  counter_h_in_y2r = counter_h_in_y2r + 1;
                end
            end
        end
    end
  endtask
  /************************************************************************
  Task to check height of output image
  *************************************************************************/
  task done_processing;
    begin
      if (counter_v_y2r == HEIGHT)
        begin
          final_one;
        end
    end
  endtask
  task final_one;
    begin
      repeat (100)
        @(posedge Clock)
        ;
      YCbCr_into_memory();
      repeat (3)
        @(posedge Clock)
        ;
      write_into_file();
      repeat (2)
        @(posedge Clock)
        ;
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                             SIMULATION COMPLETED                           <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $finish;
    end
  endtask
  /************************************************************************
  Process to write the output RGB into a memory
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (dataValidOut_y2r)
            begin
              Mem_B[k_y2r] <= Blue_out;
              Mem_G[k_y2r] <= Green_out;
              Mem_R[k_y2r] <= Red_out;
              k_y2r <= k_y2r + 1;
            end
        end
    end
  task YCbCr_into_memory;
    begin
      while ((m != MAX_SIZE_24) && (dataValidOut_y2r == 0) && (counter_v_y2r == HEIGHT))
        begin
          Mem_final[l] = Mem_B[m];
          Mem_final[l + 1] = Mem_G[m];
          Mem_final[l + 2] = Mem_R[m];
          l = l + 3;
          m = m + 1;
        end
    end
  endtask
  /************************************************************************
          Process to write the output RGB into a external file
  *************************************************************************/
  task write_into_file;
    begin
      $writememh("Enhanced_Image_out.txt", Mem_final);
    end
  endtask
  /************************************************************************
  *************Process to initiate the testbench***************************
  *************************************************************************/
  initial
    begin
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                       PROCESSING OF IMAGE STARTED                          <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      forever
        @(posedge Clock)
        begin
          read_image_delay_after_each_line_rgb;
        end
    end
  initial
    begin
      forever
        @(posedge Clock)
        begin
          read_image_delay_after_each_line_bright;
        end
    end
  initial
    begin
      forever
        @(posedge Clock)
        begin
          read_image_delay_after_each_line_y2r;
        end
    end
  /************************************************************************
  Component instantiation
  *************************************************************************/
  RGB2YCbCr #(.g_RGB_DATA_BIT_WIDTH(g_RGB_DATA_BIT_WIDTH), .g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH)) R2Y_Converter_0(.CLOCK_I(Clock),
                                                                                                                            .RESET_N_I(Reset),
                                                                                                                            .Red_i(Red),
                                                                                                                            .Green_i(Green),
                                                                                                                            .Blue_i(Blue),
                                                                                                                            .data_enable_i(dataValidIn_rgb),
                                                                                                                            .horz_sync_i(H_sync_in),
                                                                                                                            .vert_sync_i(V_sync_in),
                                                                                                                            .hactive_i(H_active_in),
                                                                                                                            .vactive_i(V_active_in),
                                                                                                                            .Y_out_o(Y_out),
                                                                                                                            .Cb_out_o(Cb_out),
                                                                                                                            .Cr_out_o(Cr_out),
                                                                                                                            .data_enable_o(dataValidOut_rgb),
                                                                                                                            .horz_sync_o(H_sync_out),
                                                                                                                            .vert_sync_o(V_sync_out),
                                                                                                                            .hactive_o(H_active_out),
                                                                                                                            .vactive_o(V_active_out));
  sin_cos_write sin_cos_write_0(.clock(Clock),
                                .nreset(Reset),
                                .write_en(write_en),
                                .write_addr(write_addr),
                                .sin_value_o(sin_value_o),
                                .cos_value_o(cos_value_o));
  DisplayEnhancements #(.g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH), .g_B_C_SAT_CONSTANT_WIDTH(g_B_C_SAT_CONSTANT_WIDTH), .g_HUE_CONSTANT_WIDTH(g_HUE_CONSTANT_WIDTH), .g_SIN_COS_DWIDTH(g_SIN_COS_DWIDTH), .g_SIN_COS_MEM_DEPTH(g_SIN_COS_MEM_DEPTH)) bright_0(.VIDEO_CLOCK_I(Clock),
                                                                                                                                                                                                                                                                     .mem_write_clock(w_clk),
                                                                                                                                                                                                                                                                     .RESET_N_I(Reset),
                                                                                                                                                                                                                                                                     .brightness_i(brightness),
                                                                                                                                                                                                                                                                     .contrast_i(contrast),
                                                                                                                                                                                                                                                                     .saturation_i(saturation),
                                                                                                                                                                                                                                                                     .hue_i(hue),
                                                                                                                                                                                                                                                                     .sin_write_en_i(sin_write_en_i),
                                                                                                                                                                                                                                                                     .cos_write_en_i(cos_write_en_i),
                                                                                                                                                                                                                                                                     .sin_write_addr_i(sin_write_addr_i),
                                                                                                                                                                                                                                                                     .cos_write_addr_i(cos_write_addr_i),
                                                                                                                                                                                                                                                                     .sin_value_i(sin_value_i),
                                                                                                                                                                                                                                                                     .cos_value_i(cos_value_i),
                                                                                                                                                                                                                                                                     .Y_i(Y_i),
                                                                                                                                                                                                                                                                     .Cb_i(Cb_i),
                                                                                                                                                                                                                                                                     .Cr_i(Cr_i),
                                                                                                                                                                                                                                                                     .data_enable_i(dataValidIn_bright),
                                                                                                                                                                                                                                                                     .horz_sync_i(horz_sync_i),
                                                                                                                                                                                                                                                                     .vert_sync_i(vert_sync_i),
                                                                                                                                                                                                                                                                     .hactive_i(hactive_i),
                                                                                                                                                                                                                                                                     .vactive_i(vactive_i),
                                                                                                                                                                                                                                                                     .Y_out_o(Yout),
                                                                                                                                                                                                                                                                     .Cb_out_o(Cbout),
                                                                                                                                                                                                                                                                     .Cr_out_o(Crout),
                                                                                                                                                                                                                                                                     .data_enable_o(dataValidOut_bright),
                                                                                                                                                                                                                                                                     .horz_sync_o(horz_sync_o),
                                                                                                                                                                                                                                                                     .vert_sync_o(vert_sync_o),
                                                                                                                                                                                                                                                                     .hactive_o(hactive_o),
                                                                                                                                                                                                                                                                     .vactive_o(vactive_o));
  YCbCr2RGB #(.g_RGB_DATA_BIT_WIDTH(g_RGB_DATA_BIT_WIDTH), .g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH)) Y2R_Converter_0(.CLOCK_I(Clock),
                                                                                                                            .RESET_N_I(Reset),
                                                                                                                            .Y_i(Y),
                                                                                                                            .Cb_i(Cb),
                                                                                                                            .Cr_i(Cr),
                                                                                                                            .data_enable_i(dataValidIn_y2r),
                                                                                                                            .horz_sync_i(H_sync_in),
                                                                                                                            .vert_sync_i(V_sync_in),
                                                                                                                            .hactive_i(H_active_in),
                                                                                                                            .vactive_i(V_active_in),
                                                                                                                            .Red_out_o(Red_out),
                                                                                                                            .Green_out_o(Green_out),
                                                                                                                            .Blue_out_o(Blue_out),
                                                                                                                            .data_enable_o(dataValidOut_y2r),
                                                                                                                            .horz_sync_o(H_sync_out),
                                                                                                                            .vert_sync_o(V_sync_out),
                                                                                                                            .hactive_o(H_active_out),
                                                                                                                            .vactive_o(V_active_out));
endmodule
